#ifndef DETECT_H
#define DETECT_H

#include <sys/types.h>
#include "detect_config.h"
#include <linux/genhd.h>

#define AUTHOR			"adussart@mandrakesoft.com"

/*Configurable path */
#define PATH_PCMCIA_LIST	"/etc/pcmcia.lst"
#define PATH_PROC_PCI		"/proc/bus/pci/devices"
#define PATH_PROC_CPUINFO	"/proc/cpuinfo"
#define PATH_PROC_MEMINFO	"/proc/meminfo"
#define	PATH_PROC_SCSI		"/proc/scsi/scsi"

/*Errors messages */
#define MALLOC_ERROR	"Can't allocate memory!\n"
#define FOPEN_ERROR	"Can't open file: %s\n"
#define KERNEL_ERROR	"You must have a kernel version superior at 2.1.120\n"
#define WEIRDNESS_ERROR "Unexpected information from /proc, please contact developers\n"


#define PCMCIA_UNKNOWN	       -1
#define PCMCIA_MULTIFUNCTION	0
#define PCMCIA_MEMORY		1
#define PCMCIA_SERIAL		2
#define PCMCIA_PARALLEL		3
#define PCMCIA_FIXEDDISK	4
#define PCMCIA_VIDEO		5
#define PCMCIA_ETHERNET		6
#define PCMCIA_AIMS		7
#define PCMCIA_SCSI		8


/*Soundcard options*/
#define HAS_OPL3           1
#define HAS_MPU401     (1<<1)
#define HAS_DMA16      (1<<2)
#define HAS_DMA2       (1<<3) 
#define HAS_AWE        (1<<4)
#define HAS_DREAM	   (1<<5)	
				
#define CPU_VENDOR		"vendor_id"
#define CPU_MODEL		"model name"


#define SECTOR_SIZE 512


typedef enum {
  UNKNOWN_BUS,
  IDE,
  ISA,
  PCI,
  PCMCIA,
  SERIAL,
  PS2,
  VLB,
  SCSI,
  FDC			/*Floppy drive controller*/		
} BusType;


typedef enum {
  UNKNOWN_DEVICE,
  CPU,
  MEMORY,
  FLOPPY,
  CDROM,
  DISK,
  ETHERNETCARD,
  SOUNDCARD,
  SCSICARD,
  VIDEOCARD,
  MOUSE,
  MODEM,
  PRINTER,
  OTHERS
} DeviceType;


typedef enum {
 MODEM_V32,	/* Modem can do V.32 (9600 bps)             */
 MODEM_V32B,	/* Modem can do V.32bis (14400 bps)         */
 MODEM_V34,	/* Modem can do V.34 (33600 bps)            */
 MODEM_V34S,	/* Modem can do V.34S (symmetrical-only)    */
 MODEM_V34B,	/* Modem can do V.34B (extended asymmetric) */
 MODEM_V34BS,	/* Modem can do V.34BS (extended symmetric) */
 MODEM_VX2,	/* Modem can do V.X2 (older 56k)            */
 MODEM_V90,	/* Modem can do V.90 (56k)                  */
 MODEM_FAX1,	/* Modem can do FAX Class 1                 */
 MODEM_FAX2,	/* Modem can do FAX Class 2                   */
 MODEM_MNP5,	/* Modem can do MNP5                          */
 MODEM_V42BIS	/* Modem can do V.42bis                       */
} ModemType;



typedef enum {
  PRI,
  LOG,
  PRILOG,
  UNUSABLE,
  USED
} PUsage;



typedef enum {
  H_SCSI,
  H_IDE
} HType;


typedef union {
    struct {
	unsigned char align[2];
	unsigned char b[SECTOR_SIZE];
    } c;
    struct {
	unsigned char align[2];
	unsigned char buffer[0x1BE];
	struct partition part[4];
	unsigned short flag;
    } p;
} PartitionTable;



typedef enum {
  PRIMARY,
  EXTENDED,
  LOGICAL,
  FREESPACE
} PType;




/*Definition of CPU */
struct cpu_info {
  unsigned short proc_id;	/*processor number */
  char *vendor;		        /*vendor name of CPU */
  char *model;		        /*model name of CPU */
  char *bogomips;		/* Bogomips */
  char *bugs;			/* Bugs detected in the micro */
  int freq;			/*frequency of CPU */
  char *cache;		/* cache size */
  int fpu;			/*FPU is present? */
  int mmx;			/*MMX is present? */
  int tdnow;		/*3dNow! is present? */
  struct cpu_info *next;
};

/*Definition of memory */
struct memory_info {
  int total;			/*total amount of memory */
  int free;			/*total free amount of memory */
  int shared;			/*total shared amount of memory */
  int buffers;		        /*total buffers */
  int cached;			/*total cached amount of memory */
  int swap_total;
  int swap_free;
};


/*Definition of PCI card */
struct pci_info {
  DeviceType type;	        /*type of the PCI card(VIDEO, ETHERNET, SCSI, SOUND) */
  char *vendor;		        /*vendor name of the PCI card */
  char *model;		        /*model name of the PCI card */
  char *modulename;				/*module name used by kernel*/
  struct pci_info *next;	/*It's a joke? */
};


/*Generic hardware device list*/
/*Contains all ISA and PCI cards recognized by detect library*/
struct cards_lst {
  unsigned long pci_id;
  char *isa_id;
  int type;
  char *vendor;
  char *model;
  char *desc;
  char *modulename;
  int options;
  int irq[10];
  int io[10];
  int dma[10];
  int dma2[10];
  int mpu401_io[10];
  BusType bus;
  struct cards_lst *next;
};


/*Definition of ISA card */
struct isa_info {
  char *vendor;		        /*vendor name of the ISA card */
  char *model;		        /*model name of the ISA card */
  DeviceType type;	       	/*type of the ISA card */
  char *modulename;	        /*module name of the ISA card*/
  int irq[10];				/*IRQ range*/
  int io[10];				/*IO range*/
  int dma[10];				/*DMA range*/
  int dma2[10];				/*DMA2 range*/
  int mpu401_io[10];		/*MPU401 range*/
  int options;                  /*ISA card options*/
  struct isa_info *next;
};



/*Definition of IDE drive */
struct ide_info {
  char *device;		        /*device of the ide drive */
  char *model;		        /*model name of the ATAPI drive */
  DeviceType type;	        /*type of the ATAPI/IDE drive(ZIP, DISK, CDROM) */
  short heads, sectors, cylinders;
  struct ide_info *next;	/*It's a joke? */
};



/* Definition of PCMCIA cards */
struct pcmcia_info {
  char *info;
  char *vendor;
  char *model;
  char *module;
  char function;
  struct pcmcia_info *next;
};



/*Definition of partitions */
struct partition_info {
  char *device;
  PType ptype;
  unsigned char sys_ind;
  unsigned char boot_ind;
  int pnum;
  int first_sector;
  int last_sector;
  int offset;
  char *label;
  char *type;			/*type indentifier of the partition */
  int size;
  struct partition_info *prev, *next, *lplist;
};


/*Definition of disk */
struct disk_info {
  char *device;
  char *model;		                /*Model name of the disk */
  char *path;
  short heads, sectors, cylinders;	/*heads, sectors and cylinders of the disk */
  short host, did;
  HType htype;
  struct partition_info *plist;
  char changed;
  BusType bus;
  int size;
  struct disk_info *next;
};


/*Definition of mouse */
struct mouse_info {
  char *device;		       /*device of the mouse */
  BusType bus;                 /*bus type of the mouse*/
  struct mouse_info *next;
};

/*Definition of modem */
struct modem_info {
  char *device;		        /*device of the modem */
  BusType bus;			/*bus type of the modem */
  long portspeed;		/*maximum serial port speed */
  short capabilities;		/*modem capabilities (V.90 etc) */
  struct modem_info *next;
};


/*Definition of SCSI card */
struct scsicard_info {
  char *vendor;		        /*vendor name of the SCSI card */
  char *model;		        /*model name of the SCSI card */
  char *module;		        /*kernel module of the SCSI card */
  BusType bus;			/*Bus of the card(ISA, PCI) */
  struct scsicard_info *next;
};

/*Definition of cdrom drive */
struct cdrom_info {
  char *device;		        /*device of the cdrom drive */
  char *model;		        /*model of the cdrom drive */
  BusType bus;                  /*bus type of the cdrom drive*/
  struct cdrom_info *next;
};


/*Definition of floppy drive */
struct floppy_info {
  char *vendor;				/*vendor name of the floppy drive*/
  char *device;		        /*device of the floppy drive */
  char *model;                  /*model of the floppy drive*/
  BusType bus;			/*bus type of the floppy drive */
  struct floppy_info *next;
};


/*Definition of VIDEO card */
struct video_info {
  char *vendor;		        /*vendor name of the video card */
  char *model;		        /*model name of the video card */
  char *server;		        /*X server name of the video card */
  BusType bus;                  /*bus type of the video card*/
  int memory;			/*Video card memory in Kb(default 256Kb) */
  char *ramdac;		        /*Video card RamDAC */
  char *clockchip;		/*Video card ClockChip */
  struct video_info *next;
};

/*Definition of ETHERNET card */
struct ethernet_info {
  char *vendor;		        /*vendor name of the ethernet card */
  char *model;		        /*model name of the ethernet card */
  char *module;		        /*kernel module name of the ethernet card */
  BusType bus;                  /*bus type of the ethernet card*/
  struct ethernet_info *next;
};


/*Definition of SOUND card */
struct soundcard_info {
  char *vendor;		        /*vendor name of the soundcard */
  char *model;                  /*model name of the soundcard */
  char *module;		        /*kernel module name of the soundcard */
  int options;                  /*soundcard options*/
  int irq[10];
  int io[10];
  int dma[10];
  int dma2[10];
  int mpu401_io[10];
  BusType bus;                  /*bus type of the soundcard*/
  struct soundcard_info *next;
};

extern char s_unknown[];
extern char s_not_available[];

/*Debug functions*/
extern int report_detect(char *filename);	/*Make a report file*/

/*Detection functions*/
extern struct memory_info *memory_detect(void);		/*Return informations about memory */
extern struct cpu_info *cpu_detect(void);	/*Return informations about CPU */
extern struct disk_info *disk_detect(struct ide_info *ide);	/*Return informations about disk-geometry */
extern struct partition_info *partitions_detect(void);	/*Return informations about partitions */
extern struct ide_info *ide_detect(void);	/*Return informations about ide drives */
extern struct mouse_info *mouse_detect(void);	/*Return informations about mouse */
extern struct modem_info *modem_detect(struct isa_info *isa);	/*Return informations avour modem */
extern struct pci_info *pci_detect(struct cards_lst *lst);	/*Return informations about PCI cards */
extern struct isa_info *isa_detect(struct cards_lst *lst);	/*Return informations about ISA cards */
extern struct pcmcia_info *pcmcia_detect(void);		/*Return informations about PCMCIA cards */
extern struct scsicard_info *scsicard_detect(struct pci_info *pci, struct isa_info *isa);	/*Return informations about SCSI card */
extern struct cdrom_info *cdrom_detect(struct ide_info *ide);	/*Return informations about CDROM */
extern struct floppy_info *floppy_detect(struct ide_info *ide);		/*Return informations about CDROM */
extern struct video_info *video_detect(struct pci_info *pci);	/*Return informations about VIDEO card  */
extern struct ethernet_info *ethernet_detect(struct pci_info *pci, struct isa_info *isa);	/*Return informations about ETHERNET card */
extern struct soundcard_info *soundcard_detect(struct pci_info *pci, struct isa_info *isa);	/*Return informations about SOUND card */
extern struct pci_info *others_detect(struct pci_info *pci); /* UNKNOWN_DEVICE */

/*List function. Contains hardware device list*/
extern struct cards_lst *init_lst(char *isa_lst, char *pci_lst);

/*Miscenaleous functions. Can be used for other programs than libdetect.*/
extern int kernel_verif();
extern char *cut_word(char *word, int start, int end);
extern char *extract_word(char *string, int num, char *delim);
extern char *find_word(char *word, char *file);
extern void *my_malloc(size_t size);
#endif
